//
//  Copyright  1997,1998 Forest Edge Software, All Rights Reserved
//
//  Program:    VAX Emulator, a "Virtual VAX"
//
//  Author:     Tom Cole
//
//  Module:     arch.h
//
//  Purpose:    This header file definitions for architectural features
//		of the host environment.
//
//  History:    05/18/99	Initial creation, pulled from vax.pch


/*
 *	This header file is typically included after the system headers
 *	from vax.pch.  This should set up the macro definitions, etc.
 *	for the box we are running on.  In the compilation mechanism
 *	for the port, specify the platform as a define on the command.
 *
 *	The supported platforms are
 *
 *	MAC		    MacOS
 *  MACHTECN    MachTen (BSD under MacOS)
 *	HPUX		HP-UX 10.20
 *	WIN		    Windows 98 or Windows NT
 *	LINUX86		Intel-based Linux systems
 *  FREEBSD		Intel-based FreeBSD systems
 *	VMS		    Alpha (!) VMS
 */

#define UNKNOWN_ARCH 1

#if defined( MAC ) || defined( macintosh )
#ifndef MAC    /* CodeWarrior gives us "macintosh" */
#define MAC 1
#endif
#undef UNKNOWN_ARCH
#define BIGENDIAN	1
typedef long long __int64;
#endif



#ifdef VMS
#define BIGENDIAN	0
#undef UNKNOWN_ARCH
#endif



#ifdef HPUX		
#define BIGENDIAN	1
#undef UNKNOWN_ARCH
typedef long long __int64;
#endif



#ifdef WIN
#define BIGENDIAN	0
#define INVERTEDLONGCHAR	0  /* VSS seems to want 0 here */
/* typedef long long __int64; */
#undef UNKNOWN_ARCH
#endif



#ifdef FREEBSD
#define BIGENDIAN	0
typedef long long __int64;
#undef UNKNOWN_ARCH
#endif


#ifdef LINUX86
#define BIGENDIAN 0
typedef long long __int64;
#undef UNKNOWN_ARCH
#endif


#ifdef MACHTEN
#define BIGENDIAN 1
typedef long long __int64;
#undef UNKNOWN_ARCH
#endif

#ifdef UNKNOWN_ARCH

#error Architecture not defined!
 
You must define an architecture in your build process to specify what build
environment and/or chip architecture you are using to build eVAX.  See the
file arch.h for more information.

#endif


/*
 *	Build additional definitions from the above definitions
 */

#if BIGENDIAN && !defined( INVERTEDLONGCHAR )

#define CHAR4( ch1, ch2, ch3, ch4 )   \
  ((( unsigned long ) ch1 << 24 ) |   \
   (( unsigned long ) ch2 << 16 ) |   \
   (( unsigned long ) ch3 << 8  ) |   \
   (( unsigned long ) ch4       ))

#else

#define CHAR4( ch1, ch2, ch3, ch4 )   \
  ((( unsigned long ) ch4 << 24 ) |   \
   (( unsigned long ) ch3 << 16 ) |   \
   (( unsigned long ) ch2 << 8  ) |   \
   (( unsigned long ) ch1       ))

#endif


