//
//  Copyright (C) 1997,1998,1999 Forest Edge Software, All Rights Reserved
//
//  Program:    eVAX, a "Virtual VAX" for Mac OS Computers
//
//  Author:     Tom Cole
//
//  Module:     console_clear.c
//
//  Purpose:    This module implements the CLEAR commands.  These are used to
//              clear out states in the console, such as the break point list
//              or the symbol table.  Additionally, CLEAR MEMORY is mapped to
//              the ZERO command.
//
//  History:    08/06/97    New header format standardization
//
//              01/20/99    Added CLEAR PROFILE instruction to clear out
//                          instruction profile counter.


#include "vax.pch"
#include "console_proto.h"
#include "asmproto.h"
#include "vaxinstr.h"


long console_clear( struct VAX ** Vax, char ** P )
{
    long rc;
    struct VAX * vax;
    char * p;
    long count, n, verb, all;
    struct BREAKSTR * bp, * nextbp;
	struct INTERRUPT * ip, *nextip;

    char * pn, * saved_p;
    char b[ 32 ];
	char zero = 0;
    extern long tb_hit, tb_try, tb_pflush;
	unsigned long stringpool_addr, stringpool_len;

    vax = *Vax;
    p = *P;
    
    if( vax == 0L ) {
        rc = VAX_NOVAX;
        return rc;
    }
    
    read_verb( vax, &p, &verb );
    
    if( verb == CHAR4('A','L','L',' ') ) {
        all = 1;
        read_verb( vax, &p, &verb );
    }
    else
        all = 0;
        
    rc = VAX_OK;
    
    switch( verb ) {

    case CHAR4('S','T','R','I'):

        strcpy( b, "CONSOLE$STRINGPOOL_BASE" );
        pn = b;
        rc = get_symbol( vax, &pn, ( long * ) &stringpool_addr );
        if( rc )
            return rc;

		strcpy( b, "CONSOLE$STRINGPOOL_SIZE" );
		pn = b;
		rc = get_symbol( vax, &pn, ( long * ) &stringpool_len );
		if( rc )
			 return rc;

        strcpy( b, "CONSOLE$STRINGPOOL" );
        pn = b;
        rc = set_symbol( vax, &pn, stringpool_addr );
 
		for( n = 0; ( unsigned long ) n < stringpool_len; n++ ) {
		    rc = store_memory( vax, stringpool_addr + n, ( void * ) &zero, 1 );
			if( rc )
				return rc;
		}
       break;

    case CHAR4('T','B',' ',' '):
        
        invalidate_tb();
        tb_try = tb_hit = tb_pflush = 0L;
        break;

    case CHAR4('E','R','R','O'):
    case CHAR4('E','R','R',' '):
    
        strcpy( b, "$STATUS" );
        pn = b;
        set_symbol( vax, &pn, 0 );
        break;
        
    case CHAR4('P','R','O','F'):

        count = 0;
        for( n = 0; n < 500; n++ ) {
            pn = instruction[ n ].name;
            if( *pn == 0 )
                break;

            if( instruction[ n ].use_count ) {
                count++;
                instruction[ n ].use_count = 0;
            }
        }

        printf( "Reset %ld instruction%s\n", count,
                  count == 1 ? "" : "s"  );

        break;

            
    case CHAR4('M','E','M',' '):
    case CHAR4('M','E','M','O'):

        rc = console_zero( &vax, &p );
        break;

    case CHAR4('S','T','A','T'):

        {
            extern long total_allocated;
            extern long count_allocated;
            extern long total_freed;
            extern long count_freed;

            total_allocated = vax-> memsize;
            count_allocated = 0L;
            total_freed = count_freed = 0L;
            break;
	}
        
    case CHAR4('S','Y','M','B'):
    case CHAR4('S','Y','M','S'):
    case CHAR4('S','Y','M',' '):
    
        if( all ) {
        
            rc = clear_all_symbols( vax );
            break;
        }
        
        printf( "\tClearing individual symbols is not currently supported.\n" );
        
        break;
    
	case CHAR4('I','N','T','E'):
	case CHAR4('I','N','T',' '):

		if( all ) {
			n = 0;
			for( ip = vax-> iqueue; ip; ip = nextip ) {
				n++;
				nextip = ip-> next;
				freemem( ip );
			}
			vax-> iqueue = 0L;
			vax-> interrupt_pending = 0;
			printf( "\tCleared %ld pending interrupt%s\n", n,
				n == 1 ? "" : "s" );
			break;
		}

        rc = asm_hex( vax, &p, ( unsigned long * ) &n );
        if( rc )
            break;
        

        nextip = 0L;
		n = 0;
        for( ip = vax-> iqueue; ip; ip = nextip ) {
            
			nextip = ip-> next;
			if( ip-> code ==  n ) {
		       if( nextip == 0L ) 
			      vax-> iqueue = vax-> iqueue -> next;
	           else
		          nextip-> next = ip-> next;
			   n++;    
			   freemem( ip );
			}
            
        }

		printf( "\tCleared %ld pending interrupt%s\n", n,
				n == 1 ? "" : "s" );

        
        break;
    
    case CHAR4('B','R','E','A'):
    case CHAR4('B','R',' ',' '):
    
        if( all ) {
            
            n = 0;
            for( bp = vax-> console.breakpoint_list; bp; bp = nextbp ) {
                n++;
                nextbp = bp-> next;
                freemem( bp );
                
            }
            
            printf( "\tCleared %ld breakpoint entr%s\n", n,
				n == 1 ? "y" : "ies" );
            
            vax-> console.breakpoint_list = 0L;
            break;
        }
        
		saved_p = p;
		rc = read_verb( vax, &p, &verb );
		if( verb == CHAR4( '/','F','A','U' )) {
			verb = BREAK_FAULT;
		}
		else {
			p = saved_p;
			verb = BREAK_INSTRUCTION;
		}

        rc = asm_hex( vax, &p, ( unsigned long * ) &n );
        if( rc )
            break;
        
        nextbp = 0L;
        for( bp = vax-> console.breakpoint_list; bp; bp = bp-> next ) {
            if( bp-> kind == verb && bp-> pc == ( unsigned long ) n )
                break;
            nextbp = bp;
            
        }
        
        if( nextbp == 0L ) 
            vax-> console.breakpoint_list = vax-> console.breakpoint_list -> next;
        else
            nextbp-> next = bp-> next;
        
        freemem( bp );
        
        break;
    
    default:
        rc = VAX_UNKPARM;
        break;
    
    }


    *Vax = vax;
    *P = p;
    return rc;
}
