//
//  Copyright (C) 1997,1998,1999 Forest Edge Software, All Rights Reserved
//
//  Program:    eVAX, a "Virtual VAX" for Mac OS Computers
//
//  Author:     Tom Cole
//
//  Module:     console_include.c
//
//  Purpose:    This module implements the INCLUDE or @ command, which accesses
//              a file to be included, and pushes it on the top of the include
//              stack.
//
//              This module also implements the ASM command shell, which puts the
//              console into assembler mode, and optionally starts processing an
//              external file containing the assembler statements.
//
//  History:    02/15/98        Initial implementation
//
//

#include "vax.pch"
#include "console_proto.h"
#include "include.h"
#include "asmproto.h"

long console_include( struct VAX ** Vax, char ** P )
{
    long rc, scanning, verify;
    struct VAX * vax;
    char * p;
    long verb, asm_mode;
    extern struct INCLUDE * include;
    
    
    vax = *Vax;
    p = *P;

//  If there's no vax, then there's a terrible error.

    if( vax == 0L ) {
        rc = VAX_NOVAX;
        return rc;
    }

	scanning = 1;
	verify = vax-> console.verify;
	asm_mode = 0;

	while( scanning ) {

//  See if there is an explicit /[NO]VERIFY switch present.  If so, use
//  the appropriate value.  Otherwise, use the default console case.
//  Note that /LIST is a synonym for /VERIFY to support the ASM case
//  that uses the include processor.

		read_verb( vax, &p, &verb );
		if( verb == CHAR4('/','V','E','R') || 
	        verb == CHAR4('/','V',' ',' ') || 
	        verb == CHAR4('/','L','I','S') ) {
			*P = p;
	        verify = 1;
	    }
	    else
		if( verb == CHAR4('/','N','O','V') || verb == CHAR4('/','N','O','L') ) {
			*P = p;
			verify = 0;
		}
		else
		if( verb == CHAR4('/','A','S','M')) {
			*P = p;
			asm_mode = 1;
		}
	    else {
			scanning = 0;
	        p = *P;
	    }
	}

//  Push the file.

    rc = push_include( vax, p, asm_mode ? ".asm" : ".com" );

//  Set the verify flag if the push worked okay.  Then we're done.

    if( rc == VAX_OK ) {
        p = p + strlen( p );
        include-> verify = verify;
    }
    
    *P = p;
    
    return rc;
}


//
//  The ASM command is sort of a special case of an INCLUDE operation.  If you
//  put ASM with no text after it, it puts the console into assembler mode.  If
//  you specify a file name, it will process that file as an assembler file.

long console_asm( struct VAX ** Vax, char ** P )
{

    struct VAX * vax;
    char * p;
    long rc;
    char buff[ 256 ], *bp;

    vax = *Vax;
    if( vax == 0L )
        return VAX_NOVAX;

//  See if there's a name on the line.  If not, then just turn on ASM mode and
//  we're done for now.

    p = *P;
    flush_blanks( &p );
    if( isend( vax, *p )) {
        vax-> console.assembler_mode = 1;
        return VAX_OK;
    }

//  Pass the command as if it was an INCLUDE statement, such that /VERIFY, etc.
//  are supported.

	strcpy( buff, " /ASM " );
	strcat( buff, *P );
	bp = &( buff[ 0 ]);

    rc = console_include( Vax, &bp );
    if( rc != VAX_OK )
        vax-> console.assembler_mode = 0;
    else {
		while( !isend( vax, **P ))
			(*P)++;
		vax-> console.assembler_mode = 1;   
	}

    return rc;
}


//	IF <expression> THEN statement

long console_if( struct VAX ** theVAX, char ** P )
{

	struct VAX * vax = *theVAX;
	char * p = *P;
	long long1, rc, verb;
	char * bp;
    char cmd[ 256 ];

   if( vax == 0L )
        return VAX_NOVAX;

	rc = asm_expr( vax, &p, &long1 );
    if( rc )
		return rc;

	flush_blanks( &p );
	bp = p;

	read_verb( vax, &p, &verb );
	if( verb != CHAR4('T','H','E','N') )
		p = bp;

	if( long1  ) {

		strcpy( cmd, p );
		rc = console_dispatch( &vax, cmd );
	}
	else
		rc = VAX_OK;

	while( !isend( vax, *p ))
		p++;

	*P = p;

	return rc;

}
