//
//	Copyright (C) 1997,1998,1999 Forest Edge Software, All Rights Reserved
//
//	Program:	eVAX, a "Virtual VAX" for Mac OS Computers
//
//	Author:		Tom Cole
//
//	Module:		emul_extended.c
//
//	Purpose:	Emulator handlers for EMUL instructions
//
//
//	History:	07/28/97	New header format standardization
//
//


#include "vax.pch"



//	The EMUL instruction

EMULATOR_ENTRY( emul_emul )
{

	long mulr, *mulr_a;
	long muld, *muld_a;
	long add,  *add_a;
	
	union XLONG {
		__int64 prod;
		long reg[ 2 ];
	} xlong;
	
	
	mulr_a = ( long * ) get_operand( vax, opcode, 0, OP_RD );
	if( mulr_a == 0L )
		return VAX_FAULT;
	mulr = *mulr_a;
	
	muld_a = ( long * ) get_operand( vax, opcode, 1, OP_RD );
	if( muld_a == 0L )
		return VAX_FAULT;
	muld = *muld_a;
	
	add_a = ( long * ) get_operand( vax, opcode, 2, OP_RD );
	if( add_a == 0L )
		return VAX_FAULT;
	add = *add_a;
	
	xlong.prod = ( __int64 ) mulr * ( __int64 ) muld;
	xlong.prod = xlong.prod + ( __int64 ) add;
	
	opcode-> size[ 3 ] = 8;
	
	//	Have to swap the longwords so they store correctly
	
	add = xlong.reg[ 0 ];
	xlong.reg[ 0 ] = xlong.reg[ 1 ];
	xlong.reg[ 1 ] = add;
	
	return put_operand( vax, opcode, 3, OP_WR, ( void * ) &xlong.prod );
	
}






//	The EDIV instruction

EMULATOR_ENTRY( emul_ediv )
{

	long divr, *divr_a;
	long quo, rem;
	
	union XLONG {
		__int64 prod;
		long reg[ 2 ];
	} xlong, *xlong_a;
	__int64 xvalue;
	long rc;
	
	
	
//	Get the divisor

	divr_a = ( long * ) get_operand( vax, opcode, 0, OP_RD );
	if( divr_a == 0L )
		return VAX_FAULT;
	divr = *divr_a;

//	Get the dividend.  Have to swap the longwords so the math
//	works right.

	xlong_a = ( void * ) get_operand( vax, opcode, 1, OP_RD );
	if( xlong_a == 0L )
		return VAX_FAULT;
	xlong = *xlong_a;
	rem = xlong.reg[ 0 ];
	xlong.reg[ 0 ] = xlong.reg[ 1 ];
	xlong.reg[ 1 ] = rem;
	
	if( divr == 0L ) {
	
		quo = xlong.reg[ 1 ];
		rem = 0L;
	}
	else {
	
		quo = xlong.prod / divr;
		xvalue = ( __int64 ) quo * ( __int64 ) divr;
		rem = ( long ) xlong.prod - xvalue;
	}

//	Store the operands

	rc = put_operand( vax, opcode, 2, OP_WR, ( void * ) &quo );
	if( rc == VAX_OK )
		rc = put_operand( vax, opcode, 3, OP_WR, ( void * ) &rem );
	
	return rc;
	
}


