//
//	Copyright (C) 1997,1998,1999 Forest Edge Software, All Rights Reserved
//
//	Program:	eVAX, a "Virtual VAX" for Mac OS Computers
//
//	Author:		Tom Cole
//
//	Module:		emul_movc.c
//
//	Purpose:	Emulator handlers for MOVC3 and MOVC5 instructions
//
//	History:	07/28/97	New header format standardization
//
//




#include "vax.pch"

EMULATOR_ENTRY( emul_movc3 )
{

	char 	src1;

	short 	len, *len_a;

	long	tmp1, tmp2, tmp3, tmp4;

//	See if the addresses are registers

	if( opcode-> is_register[ 1 ] || opcode-> is_register[ 2 ])
		return set_fault( vax, EXC_RESOP, 0 );

//	Get addressability to the length.

	len_a = ( short * ) get_operand( vax, opcode, 0, OP_RD );
	if( !len_a )
		return VAX_FAULT;
	len = *len_a;

//	Load up the working values

	tmp1 = len;
	tmp2 = opcode-> VAXaddr[ 1 ];
	tmp3 = opcode-> VAXaddr[ 2 ];

//	We have to handle overlap correctly, so there are really two versions of the loop.

	if( tmp2  > tmp3 ) {
	
		while( tmp1 > 0 ) {
			
			load_memory( vax, tmp2, ( void * ) &src1, 1 );
			store_memory( vax, tmp3, ( void * ) &src1, 1 );
			tmp1--;
			tmp2++;
			tmp3++;
		}
		
		vax-> R1 = tmp2;
		vax-> R3 = tmp3;
		
	}
	else {
	
		tmp4 = tmp1;
		tmp2 = tmp2 + tmp1;
		tmp3 = tmp3 + tmp1;
		
		while( tmp1 > 0 ) {
			tmp1--;
			tmp2--;
			tmp3--;
			load_memory( vax, tmp2, ( void * ) &src1, 1 );
			store_memory( vax, tmp3, ( void * ) &src1, 1 );
		}
		
		vax-> R1 = tmp2 + tmp4;
		vax-> R3 = tmp3 + tmp4;
	}
	
	vax-> R0 = 0L;
	vax-> R2 = 0L;
	vax-> R4 = 0L;
	vax-> R5 = 0L;

	vax-> pslw.v = 0;
	vax-> pslw.n = 0;
	vax-> pslw.z = 1;
	vax-> pslw.c = 0;
	
	return VAX_OK;
}




EMULATOR_ENTRY( emul_movc5 )
{

	char 	src1, fill, *fill_a;

	short 	len1, len2, *len_a;

	long	tmp1, tmp2, tmp3, tmp4, tmp5, tmp6;

//	See if the addresses are registers

	if( opcode-> is_register[ 1 ] || opcode-> is_register[ 3 ])
		return set_fault( vax, EXC_RESOP, 0 );

//	Get addressability to the source length.

	len_a = ( short * ) get_operand( vax, opcode, 0, OP_RD );
	if( !len_a )
		return VAX_FAULT;
	len1 = *len_a;

//	Get addressability to the destination length.

	len_a = ( short * ) get_operand( vax, opcode, 3, OP_RD );
	if( !len_a )
		return VAX_FAULT;
	len2 = *len_a;

//	Get the fill byte

	fill_a = ( char * ) get_operand( vax, opcode, 2, OP_RD );
	if( !fill_a )
		return VAX_FAULT;
	fill = *fill_a;

	
//	Load up working registers

	tmp1 = len1;
	tmp2 = opcode-> VAXaddr[ 1 ];
	tmp3 = len2;
	tmp4 = opcode-> VAXaddr[ 3 ];

//	We have to handle overlap correctly, so there are really two versions of the loop.

	if( tmp2  > tmp4 ) {
	
		while( tmp1 != 0 && tmp3 != 0 ) {
			
			load_memory( vax, tmp2, ( void * ) &src1, 1 );
			store_memory( vax, tmp4, ( void * ) &src1, 1 );
			tmp1--;
			tmp2++;
			tmp3--;
			tmp4++;
		}
		
		while( tmp3 != 0 ) {
			store_memory( vax, tmp4, ( void * ) &fill, 1 );
			tmp3--;
			tmp4++;
		}
	
		vax-> R1 = tmp2;
		vax-> R3 = tmp4;
	}
	else {
	
		tmp5 = ( ( unsigned long ) tmp1 < ( unsigned long ) tmp3 ) ? tmp1 : tmp3;
				/* MINU( tmp1, tmp3 ) */
				
		tmp6 = tmp3;
		tmp2 = tmp2 + tmp5;
		tmp4 = tmp4 + tmp6;
		
		while( tmp3 > tmp1 ) {
			tmp3--;
			tmp4--;
			store_memory( vax, tmp4, ( void * ) &fill, 1 );
		}
		
		while( tmp3 != 0 ) {
			tmp1--;
			tmp2--;
			tmp3--;
			tmp4--;
			load_memory( vax, tmp2, ( void * ) &src1, 1 );
			store_memory( vax, tmp4, ( void * ) &src1, 1 );
		}
		
		vax-> R1 = tmp2 + tmp5;
		vax-> R3 = tmp4 + tmp6;
	}
	
	vax-> R0 = tmp1;
	vax-> R2 = 0;
	vax-> R4 = 0;
	vax-> R5 = 0;
	
	vax-> pslw.n = ( len1 < len2 );
	vax-> pslw.z = ( len1 == len2 );
	vax-> pslw.v = 0;
	vax-> pslw.c = ( ( unsigned short ) len1 < ( unsigned short ) len2 );
		
	return VAX_OK;
}


